#include "RunGame.h"

RunGame::RunGame()
{
	memset(left_global_pos, 0, 3 * sizeof(float));

	memset(right_global_pos, 0, 3 * sizeof(float));

	memset(result, 0, 4 * sizeof(int));

	down_wait_after_jump = -1;

	last_left_zupt = 1;
	last_right_zupt = 1;
}

void RunGame::setData(deque<shoes_data_cell>& shoes_data_vector, int time_stamp, float pos_x, float pos_y, float pos_z, float heading, float pitch, float roll, int zupt, int rssi)
{
	if (shoes_data_vector.size() > 0)
	{
		if (shoes_data_vector.front().zupt == 0)
		{
			shoes_data_vector.clear();
		}
		else if (shoes_data_vector.back().zupt == 1 && zupt == 1)
		{
			shoes_data_vector.clear();
		}
	}
	shoes_data_vector.push_back({ time_stamp,  pos_x,  pos_y,  pos_z, heading, pitch, roll, zupt, rssi });
}

void RunGame::calGlobalPos(deque<shoes_data_cell>& shoes_data_vector, deque<shoes_data_cell>& step_data_vector, float* global_pos, int LEFT_OR_RIGHT)
{
	int length = shoes_data_vector.size();

	if (length == 0)
	{
		memset(global_pos, 0, 3 * sizeof(float));
		step_data_vector.clear();

		return;
	}

	shoes_data_cell shoes_data_temp = shoes_data_vector.back();

	if (length == 1)
	{
		step_data_vector.clear();

		shoes_data_temp.pos_x = 0;
		shoes_data_temp.pos_y = 0;
		shoes_data_temp.pos_z = 0;

		step_data_vector.push_back(shoes_data_temp);

		memset(global_pos, 0, 3 * sizeof(float));

		return;
	}

	float pos_offset[3];

	pos_offset[0] = shoes_data_vector.at(length - 1).pos_x - shoes_data_vector.at(length - 2).pos_x;
	pos_offset[1] = shoes_data_vector.at(length - 1).pos_y - shoes_data_vector.at(length - 2).pos_y;
	pos_offset[2] = shoes_data_vector.at(length - 1).pos_z - shoes_data_vector.at(length - 2).pos_z;

	float posTemp[3];

	float heading = shoes_data_vector[0].heading;

	posTemp[0] = cos(heading) * pos_offset[0] + sin(heading) * pos_offset[1];
	posTemp[1] = -sin(heading) * pos_offset[0] + cos(heading) * pos_offset[1];


	if (LEFT_OR_RIGHT == LEFT_FOOT)
	{
		/*
		*  ΪЬЬӳģ²򣬻ʹƵX켣䳤
		*/
		pos_offset[0] = 0.9848 * posTemp[0] - 0.1736 * posTemp[1];
		pos_offset[1] = 0.1736 * posTemp[0] + 0.9848 * posTemp[1];
	}
	else
	{
		pos_offset[0] = 0.9848 * posTemp[0] + 0.1736 * posTemp[1];
		pos_offset[1] = -0.1736 * posTemp[0] + 0.9848 * posTemp[1];
	}

	for (int i = 0; i < 3; i++)
	{
		global_pos[i] += pos_offset[i];
	}

	shoes_data_temp.pos_x = step_data_vector.back().pos_x + pos_offset[0];
	shoes_data_temp.pos_y = step_data_vector.back().pos_y + pos_offset[1];
	shoes_data_temp.pos_z = step_data_vector.back().pos_z + pos_offset[2];

	step_data_vector.push_back(shoes_data_temp);
}

void RunGame::setPolarAccUnzupt(int* max_acc, int* min_acc, int* acc, int zupt)
{
	if (zupt != 1)
	{
		for (int i = 0; i < 3; i++)
		{
			if (max_acc[i] < acc[i])
			{
				max_acc[i] = acc[i];
			}

			if (min_acc[i] > acc[i])
			{
				min_acc[i] = acc[i];
			}
		}
	}
	else
	{
		for (int i = 0; i < 3; i++)
		{
			max_acc[i] = acc[i];
			min_acc[i] = acc[i];
		}
	}

}

bool RunGame::unzuptValid(int* max_acc, int* min_acc)
{
	if (max_acc[0] - min_acc[0] > 1024 || max_acc[1] - min_acc[1] > 1024 || max_acc[2] - min_acc[2] > 1024)
	{
		return true;
	}

	return false;
}

void RunGame::Process(int time_stamp, int* right_pos, int* right_att, int* right_acc, int right_zupt, int right_front_press, int right_back_press,
	int* left_pos, int* left_att, int* left_acc, int left_zupt, int left_front_press, int left_back_press,
	int jump, int down, int rssi)
{
	setData(right_shoes_data_vector, time_stamp, right_pos[0] * 0.001f, right_pos[1] * 0.001f, right_pos[2] * 0.001f,
		right_att[0] * 0.0001f, right_att[1] * 0.0001f, right_att[2] * 0.0001f, right_zupt, rssi);

	setData(left_shoes_data_vector, time_stamp, left_pos[0] * 0.001f, left_pos[1] * 0.001f, left_pos[2] * 0.001f,
		left_att[0] * 0.0001f, left_att[1] * 0.0001f, left_att[2] * 0.0001f, left_zupt, rssi);

	calGlobalPos(right_shoes_data_vector, right_step_data_vector, right_global_pos, RIGHT_FOOT);
	calGlobalPos(left_shoes_data_vector, left_step_data_vector, left_global_pos, LEFT_FOOT);

	//ж
	int left_cmd = -1;
	if (left_step_data_vector.size() > 1 && left_step_data_vector.front().zupt == 1)
	{

		//if (left_step_data_vector.front().zupt == 1 && left_step_data_vector.back().zupt == 1)
		//{
		//	/*for (int i = 0; i < left_step_data_vector.size(); i++)
		//	{
		//		std::cout << "left_pos_x : " << left_step_data_vector[i].pos_x << "  pitch : " << left_step_data_vector[i].pitch << endl;
		//	}
		//	std::cout << endl;*/

		//	// ͳƲ
		//	/*if (unzuptValid(max_acc_unzupt_left, min_acc_unzupt_left))
		//	{
		//		addMotionCount(STEP_COUNT);
		//	}*/
		//}
		if (left_step_data_vector.back().zupt || left_step_data_vector.back().pitch < 0.3f)
		{

			float max_val = left_step_data_vector.front().pos_x;

			for (int i = 0; i < left_step_data_vector.size(); i++)
			{
				if (left_step_data_vector[i].pos_x > max_val)
				{
					max_val = left_step_data_vector[i].pos_x;
				}
			}

			if (max_val - left_step_data_vector.back().pos_x > 0.23f)
			{
				left_cmd = MOTION_LEFT;
				std::cout << "motion: left_side" << std::endl;

				left_step_data_vector.pop_front();
			}
		}
	}

	int right_cmd = -1;
	if (right_step_data_vector.size() > 1 && right_step_data_vector.front().zupt == 1)
	{
		// ͳƲ
		/*if (unzuptValid(max_acc_unzupt_right, min_acc_unzupt_right))
		{
			addMotionCount(STEP_COUNT);
		}*/

		if (right_step_data_vector.back().zupt || right_step_data_vector.back().pitch < 0.3f)
		{

			float min_val = right_step_data_vector.front().pos_x;

			for (int i = 0; i < right_step_data_vector.size(); i++)
			{
				if (right_step_data_vector[i].pos_x < min_val)
				{
					min_val = right_step_data_vector[i].pos_x;
				}
			}

			if (right_step_data_vector.back().pos_x - min_val > 0.23f)
			{
				right_cmd = MOTION_RIGHT;

				std::cout << "motion: right_side" << std::endl;

				right_step_data_vector.pop_front();
			}
		}
	}

	int jump_cmd = -1;

	jump_cmd = getResultJump(jump);

	int down_cmd = -1;

	down_cmd = getResultDown(down);

	result[0] = left_cmd;
	result[1] = right_cmd;
	result[2] = jump_cmd;
	result[3] = down_cmd;

	//Ʋ
	if (last_left_zupt == 0 && left_zupt == 1 && unzuptValid(max_acc_unzupt_left, min_acc_unzupt_left))
	{
		addMotionCount(STEP_COUNT);
	}

	if (last_right_zupt == 0 && right_zupt == 1 && unzuptValid(max_acc_unzupt_right, min_acc_unzupt_right))
	{
		addMotionCount(STEP_COUNT);
	}

	setPolarAccUnzupt(max_acc_unzupt_left, min_acc_unzupt_left, left_acc, left_zupt);
	setPolarAccUnzupt(max_acc_unzupt_right, min_acc_unzupt_right, right_acc, right_zupt);

	last_left_zupt = left_zupt;
	last_right_zupt = right_zupt;

}

int RunGame::getResultDown(int down)
{
	int isDown = -1;


	if (last_down == 0 && down == 1 && down_wait_after_jump < 0)
	{
		isDown = MOTION_DOWN;

		addMotionCount(DOWN_COUNT);

		std::cout << "motion : down" << std::endl;

		down_count++;
	}
	/*
	* down_wait_after_jump:ǰŶ׺󣬽źѹdown_wait_after_jump == -1ʱж϶Ч
	* һֱڶ׵ʱ򣬶ʱ䣬ʱ䲻ס
	*/
	if (down && down_wait_after_jump < 0)
	{
		down_wait_after_jump = 10;
	}

	if (down == 0 && down_wait_after_jump >=0)
	{
		down_wait_after_jump--;
	}

	last_down = down;

	return isDown;

}

int RunGame::getResultJump(int jump)
{
	int isJump = -1;

	if (last_jump == 0 && jump == 1)
	{
		isJump = MOTION_JUMP;

		addMotionCount(JUMP_COUNT);

		std::cout << "motion : jump" << std::endl;

		jump_count++;
	}
	/*
	* ֮Ŷףdown_wait_after_jump
	*/
	if (jump)
	{
		down_wait_after_jump = 10;
	}

	last_jump = jump;

	return isJump;
}



void RunGame::setStepHeading(float& cur_heading, float heading)
{
	cur_heading = heading;
}

void RunGame::getResult(int* dec)
{
	memcpy(dec, result, 4 * sizeof(int));
}

float RunGame::getGamePos(int left_or_right, int index)
{
	if (index < 0 || index > 2)
		return -1;

	if (left_or_right == LEFT_FOOT)
	{
		return left_global_pos[index];
	}
	else
	{
		return right_global_pos[index];
	}
}