#include "TestGame.h"

TestGame::TestGame()
{
	memset(left_global_pos, 0, 3 * sizeof(float));
	memset(right_global_pos, 0, 3 * sizeof(float));

	memset(left_cmd_global_pos, 0, 3 * sizeof(float));
	memset(right_cmd_global_pos, 0, 3 * sizeof(float));

	memset(result, 0, 4 * sizeof(int));

	last_left_zupt = 1;

	last_right_zupt = 1;

	left_step = 0;
	right_step = 0;

	left_step_length = 0;
	right_step_length = 0;

}


void TestGame::setData(deque<acc_pos_struct>& shoes_data_vector, float pos_x, float pos_y, float pos_z,
	float acc_x, float acc_y, float acc_z, float heading, float pitch, float roll, int zupt)
{
	if (shoes_data_vector.size() > 0)
	{
		if (shoes_data_vector.front().zupt == 0)
		{
			shoes_data_vector.clear();
		}
		else if (shoes_data_vector.back().zupt == 1 && zupt == 1)
		{
			shoes_data_vector.clear();
		}
	}
	
	float sin_pitch = sin(pitch);
	float cos_pitch = cos(pitch);

	float sin_roll = sin(roll);
	float cos_roll = cos(roll);

	float local_acc_x = cos_pitch * acc_x + sin_pitch * sin_roll  * acc_y - sin_pitch * cos_roll * acc_z;
	float local_acc_y = cos_roll * acc_y + sin_roll * acc_z ;
	//float local_acc_z = -sin_pitch * acc_x + cos_pitch * sin_roll * acc_y - cos_pitch * cos_roll * acc_z;

	float norm_xy = sqrt(local_acc_x * local_acc_x + local_acc_y * local_acc_y);

	//std::cout << "local_acc_x : " << local_acc_x << ", local_acc_y : " << local_acc_y << " local_acc_z : " << local_acc_z << endl;

	shoes_data_vector.push_back({  pos_x,  pos_y,  pos_z, norm_xy, zupt });
}

 void Process_motion(int time_stamp, deque<acc_pos_struct> &left_shoes_data_vector, deque<double_press_struct> &left_press_vector, int &left_step, int& left_step_length,
	 int & last_left_zupt, int &left_zupt, int left_or_right)
{
	 //10buff
	 if (left_press_vector.size() > 10)
	 {
		 left_press_vector.pop_front();
	 }

	 int press_len = left_press_vector.size();

	 int left_front_min_val = left_press_vector[0].front_press;
	 int left_back_min_val = left_press_vector[0].back_press;

	 for (int i = 1; i < press_len; i++)
	 {
		 if (left_press_vector[i].front_press < left_front_min_val)
		 {
			 left_front_min_val = left_press_vector[i].front_press;
		 }

		 if (left_press_vector[i].back_press < left_back_min_val)
		 {
			 left_back_min_val = left_press_vector[i].back_press;
		 }
	 }
	 if (left_press_vector.back().front_press > left_front_min_val + 500 || left_press_vector.back().back_press > left_back_min_val + 500)
	 {
		 left_press_vector.back().up_flag = 1;
	 }

	 if (left_press_vector.size() > 1 && left_press_vector.back().up_flag == 1)
	 {
		 left_press_vector.back().get_flag = left_press_vector[press_len - 2].get_flag;
	 }

	 if (last_left_zupt == 0 && left_zupt == 1 && left_press_vector.back().get_flag == 0 && left_press_vector.back().up_flag == 1)
	 {
		 //ѹжϰ

		 left_press_vector.back().get_flag = 1;

		 ++left_step;

		 if(left_or_right == LEFT_FOOT)
			std::cout << "times : " << time_stamp << ", left_step : " << left_step << endl;
		 else
			 std::cout << "times : " << time_stamp << ", right_step : " << left_step << endl;

		 //

		 if (left_shoes_data_vector.size() > 3 && left_shoes_data_vector.front().zupt == 1 && left_shoes_data_vector.back().zupt == 1)
		 {
			 float max_acc_xy = left_shoes_data_vector.front().acc_xy_norm;

			 int vec_len = left_shoes_data_vector.size();

			 for (int i = 1; i < vec_len - 0.5 * vec_len; i++)
			 {
				 if (max_acc_xy < left_shoes_data_vector[i].acc_xy_norm)
				 {
					 max_acc_xy = left_shoes_data_vector[i].acc_xy_norm;
				 }
			 }
			 float dx = left_shoes_data_vector.front().pos_x - left_shoes_data_vector.back().pos_x;
			 float dy = left_shoes_data_vector.front().pos_y - left_shoes_data_vector.back().pos_y;

			 if (max_acc_xy < 0.8f)
			 {
				 left_step_length = 0;
				 std::cout << "step_length : 0.0  m" << 0 << endl;
			 }
			 else
			 {
				 left_step_length = sqrt(dx * dx + dy * dy) * 1000.0f;

				 std::cout << "step_length : " << left_step_length * 0.001f << " m" << endl;
			 }
		 }
		 else
		 {
			 left_step_length = 0;
			 std::cout << "step_length : 0.0  m" << 0 << endl;
		 }

	 }


	 last_left_zupt = left_zupt;
}

void TestGame::Process(int time_stamp, int* right_pos, int* right_att, int* right_acc, int right_zupt, int right_front_press, int right_back_press,
	int* left_pos, int* left_att, int* left_acc, int left_zupt, int left_front_press, int left_back_press,
	int jump, int down, int rssi)
{

	setData(right_shoes_data_vector, right_pos[0] * 0.001f, right_pos[1] * 0.001f, right_pos[2] * 0.001f, 
		right_acc[0] / 2048.f, right_acc[1] / 2048.f, right_acc[2] / 2048.f,
		right_att[0] * 0.0001f, right_att[1] * 0.0001f, right_att[2] * 0.0001f, right_zupt);

	setData(left_shoes_data_vector, left_pos[0] * 0.001f, left_pos[1] * 0.001f, left_pos[2] * 0.001f,
		left_acc[0] / 2048.f, left_acc[1] / 2048.f, left_acc[2] / 2048.f,
		left_att[0] * 0.0001f, left_att[1] * 0.0001f, left_att[2] * 0.0001f, left_zupt);

	left_press_vector.push_back({ left_front_press, left_back_press, 0, 0});

	right_press_vector.push_back({ right_front_press, right_back_press, 0,0});

	Process_motion(time_stamp, left_shoes_data_vector, left_press_vector, left_step, left_step_length,
		last_left_zupt, left_zupt, LEFT_FOOT);

	Process_motion(time_stamp, right_shoes_data_vector, right_press_vector, right_step, right_step_length,
		last_right_zupt, right_zupt, RIGHT_FOOT);

	memset(result, 0, 4 * sizeof(int));

	if (left_step_length > right_step_length)
	{
		result[0] = left_step_length;
		result[1] = 0;
	}
	else
	{
		result[0] = right_step_length;
		result[1] = 1;
	}

	result[2] = left_step + right_step;

}



void TestGame::getResult(int* dec)
{
	memcpy(dec, result, 4 * sizeof(int));
}

float TestGame::getGamePos(int left_or_right, int index)
{
	if (index < 0 || index > 2)
		return -1;

	if (left_or_right == LEFT_FOOT)
	{
		return left_global_pos[index];
	}
	else
	{
		return right_global_pos[index];
	}
}